#!/usr/bin/python
# radius.py

"""
Simple radius packet library
"""
import sys
import socket
import select
import struct
import md5

VERSION="1.3"
AUTHOR="Evgeny Demidov"

ACCESS_REQUEST = 1
ACCESS_ACCEPT = 2
ACCESS_REJECT = 3
ACCOUNTING_REQUEST = 4
ACCOUNTING_RESPONSE = 5
ACCESS_CHALLENGE = 11

PW_USER_NAME = 1
PW_USER_PASSWORD = 2

packet_codes = [
	ACCESS_REQUEST,
	ACCESS_ACCEPT,
	ACCESS_REJECT,
	ACCOUNTING_REQUEST,
	ACCOUNTING_RESPONSE,
	ACCESS_CHALLENGE
]

# Taken from freeradius
# We dont want to bruteforce from 0 to 65535
known_vendor_ids = {
"3com" :	43,
"3GPP" :	10415,
"3GPP2" :	5535,
"Acc" : 5,
"Alcatel" : 3041,
"Alteon" : 1872,
"Altiga" : 3076,
"Aptis" :	2637,
"Ascend" :	529,
"Bay-Networks" :	1584,
"BinTec" :	272,
"Bristol" :4363,
"Cabletron" :	52,
"Cisco" :		9,
"Cisco-BBSM" : 5263,
"Cisco-VPN3000" :	3076,
"Cisco-VPN5000" :	255,
"Colubris" :                8744,
"Columbia-University" : 11862,
"ERX" :     4874,
"Extreme" :		1916,
"Foundry" :		1991,
"FreeRADIUS" :	11344,
"Gandalf" :                64,
"ITK" :	1195,
"Juniper" :		2636,
"KarlNet" : 762,
"Livingston":	307,
"Merit":	61,
"Microsoft" :	311,
"Mikrotik" :	14988,
"Navini" :          6504,
"Netscreen" :	3224,
"Nomadix" :  3309,
"Propel" : 14895,
"Quintum" :		6618,
"Redback" :		2352,
"RedCreek" :	1958,
"Shasta" :		3199,
"Shiva" :		166,
"SonicWall" :	8741,
"SpringTide" :      3551,
"Telebit" : 117,
"Trapeze" :		14525,
"Unix":	4,
"USR" :		429,
"ValemountNetworks" :	16313,
"Versanet" : 2180,
"WISPr" :        14122,
"Xedia" :           838
}

class Radius:
	def __init__(self, host, port, timeout=0.5, numpackets=-1):
		self.host = host
		self.port = port
		self.timeout = timeout
		self.numpackets = numpackets
		self.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		self.count = 1
		self.id = 0
		self.authenticator = 0

		print "Timeout for IO operations: %s seconds" % str(self.timeout)
	
	def send(self, code, data="", length=-1):
		print "Running test %d" % self.count

		if length == -1:
			length = 20 + len(data)

		packet = ""
		packet += chr(code)
		packet += chr(self.id & 0xff)
		packet += struct.pack(">h", length)
		packet += struct.pack(">l", (self.authenticator & 0xffffffff)) * 4
		packet += data

		self.sock.sendto(packet, (self.host, self.port))

		select.select([self.sock.fileno()], [], [], self.timeout)
	
		if self.numpackets > 0:
			if self.numpackets == 1:
				print "Max packet count reached"
				sys.exit()
			self.numpackets -= 1
	
		self.id += 1
		self.authenticator += 1
		self.count += 1

	def send_all(self, data="", length=-1):
		for code in range(0, 256):
			self.send(code, data, length)

	def send_valid(self, data="", length=-1):
		for code in packet_codes:
			self.send(code, data, length)

	def encode_attr(self, attr, data=""):
		s = chr(attr) + chr((len(data) + 2) & 0xff) + data
		return s

	def encode_passwd(self, secret, passwd=""):
		result = ""

		# add padding if needed
		n = len(passwd) % 16
		if len(passwd) > 0 and n > 0:
			n = 16 - n
			passwd += "\x00" * n

		md = md5.new()
		md.update(secret)
		md.update(struct.pack(">l", (self.authenticator & 0xffffffff)) * 4)	
		digest = md.digest()

		length = 16
		if len(passwd) < 16: 
			length = len(passwd)

		for i in range(0, length):
			result += chr(ord(passwd[i]) ^ ord(digest[i]))

		# extended algorithm, hope its correct
		if len(passwd) > 16:
			lastxor = result
			for n in range(16, len(passwd), 16):
				md = md5.new()
				md.update(secret)
				md.update(lastxor)
				digest = md.digest()
				lastxor = ""
				for i in range(0,16):
					lastxor += chr(ord(passwd[n+i]) ^ ord(digest[i]))	
				result += lastxor
		
		return result

	def close(self):
		self.sock.close()


